/*
 * ConfigurationParser.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			28 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes;

import java.io.*;
import java.util.*;

/**
 * This class is used for options retrieving in a configuration file and
 * arguments parsing.
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1.1
 */
public class ConfigurationParser {
	
	// Some class constants
	private static final String commentSign			= "#";
	private static final int	defaultIntValue		= 0;
	private static final float	defaultFloatValue	= 0;
			
    // Static string hash for valid options 
	private static HashMap<String, String>	configOptions;
	private static HashMap<String, String>	commandLineAliases;
	private static HashMap<String, Boolean>	lockedOption;

    /**
     * Reset all settings of the ConfigurationParser class.
     */ 
    public static void reset() {
    	
        configOptions		= new HashMap<String, String>();
        configOptions.clear();
        
        commandLineAliases	= new HashMap<String, String>();
        commandLineAliases.clear();
        
        lockedOption		= new HashMap<String, Boolean>();
        lockedOption.clear();
        
    }
    
    /**
     * Resets the ConfigurationParser class and invoke some other
     * classes methods to define valid options in config file.
     */
    public static void initialize() {
    	
    	// Reset
    	ConfigurationParser.reset();
    	
    	// Register each class options
    	br.usp.ime.linux.lemes.SquidPCB.registerConfigs();
    	br.usp.ime.linux.lemes.UrlObjectAnalyzer.UrlObjectAnalyzer.registerConfigs();
    	br.usp.ime.linux.lemes.Datastructure.DataStructureManager.registerConfigs();
    	br.usp.ime.linux.lemes.Datastructure.UrlNode.registerConfigs();
    	br.usp.ime.linux.lemes.IPC.ConnectionServer.registerConfigs();
    	br.usp.ime.linux.lemes.IPC.ConnectionThread.registerConfigs();
    	br.usp.ime.linux.lemes.SquidExternalACL.registerConfigs();
    	br.usp.ime.linux.lemes.IPC.UserAuthenticator.registerConfigs();
    	
    }
    
    /**
     * Receive the arguments from the command line and set the proper values.
     * 
     * @param commandLine The arguments.
     */
    public static void parseCommandLine(String commandLine[]) {
    	
    	int currentArgument = 0,
    		totalArguments = commandLine.length;
    	
    	while (currentArgument < totalArguments) {
    		
    		if (commandLineAliases.containsKey(commandLine[currentArgument])) {
    			if (currentArgument + 1 >= totalArguments) {
    				Logging.error("ConfigurationParser: Missing arguments for option '" + commandLine[currentArgument] + "'.");
    			}
    			else {
    				configOptions.put(commandLineAliases.get(commandLine[currentArgument]), commandLine[currentArgument + 1]);
    				lockedOption.put(commandLineAliases.get(commandLine[currentArgument]), true);
    				currentArgument += 2;
    			}
    		}
    		else if (commandLine[currentArgument].startsWith("--") &&
    				 configOptions.containsKey(commandLine[currentArgument].substring(2))) {
    			if (currentArgument + 1 >= totalArguments) {
    				Logging.error("ConfigurationParser: Missing arguments for option '" + commandLine[currentArgument] + "'.");
    			}
    			else {
    				configOptions.put(commandLine[currentArgument].substring(2), commandLine[currentArgument + 1]);
    				lockedOption.put(commandLine[currentArgument].substring(2), true);
    				currentArgument += 2;
    			}
    		}
    		else {
    			Logging.error("ConfigurationParser: Unrecognized command line option: " + commandLine[currentArgument]);
    			currentArgument++;
    		}
    	}
    	
    }

    /**
     * Define a valid option name that should be accepted from the config file.
     * 
     * @param optionName The option name.
     * @return <b>true</b> if successful or <b>false</b> on errors.
     */
    public static boolean setOptionName(String optionName) {
    	
    	if (optionName.length() == 0) {
    		Logging.debug("ConfigurationParser: Invalid option name. (zero length)");
    		return false;
    	}
    	
    	if (optionName.contains(" ")) {
    		Logging.debug("ConfigurationParser: Invalid option name: '" + optionName +
    			"' (contains spaces)");
    		return false;
    	}
    	
    	configOptions.put(optionName, null);
    	lockedOption.put(optionName, false);
    	return true;
    	
    }
    
    /**
     * Define a valid option name that should be accepted from the config file and
     * it's default value.
     * 
     * @param optionName The option name.
     * @param value The default value.
     * @return <b>true</b> if successful or <b>false</b> on errors.
     */
    public static boolean setOptionName(String optionName, String value) {
    	
    	if (optionName.length() == 0) {
    		Logging.debug("ConfigurationParser: Invalid option name. (zero length)");
    		return false;
    	}
    	
    	if (optionName.contains(" ")) {
    		Logging.debug("ConfigurationParser: Invalid option name: '" + optionName +
    			"' (contains spaces)");
    		return false;
    	}
    	
    	configOptions.put(optionName, value);
    	lockedOption.put(optionName, false);
    	return true;
    	
    }

    /**
     * Set an alias to be used in command line options.
     * 
     * @param optionName The real option name.
     * @param alias The alias for the option name below (tipically -x, where x means any_letter).
     * @return <b>true</b> if successful or <b>false</b> on errors.
     */
    public static boolean setOptionNameCommandlineAlias(String optionName, String alias) {
    	
    	if (optionName.contains(" ")) {
    		Logging.debug("ConfigurationParser: Invalid option name: '" + optionName +
    			"' (contains spaces)");
    		return false;
    	}
    	
    	if (!configOptions.containsKey(optionName)) {
    		Logging.debug("ConfigurationParser: Invalid option name for alias setting: '" + optionName +
				"' (option name must be set before the alias)");
    		return false;
    	}
    	
    	if (alias.length() == 0) {
    		Logging.debug("ConfigurationParser: Invalid alias for option name: '" + optionName +
				"' (zero length)");
    		return false;
    	}
    	
    	commandLineAliases.put(alias, optionName);
    	return true;
    	
    }
    
    /**
     * Check if any value is defined for an option name.
     * 
     * @param optionName The option name.
     * @return <b>true</b> if is defined or <b>false</b> if don't.
     */    
    public static boolean isDefined(String optionName) {
    	
    	if (configOptions.containsKey(optionName) &&
    		configOptions.get(optionName) != null)
    			return true;
    	else
    		return false;
    	
    }
    
    /**
     * Get the int value for the requested option.
     * 
     * @param optionName The option name.
     * @return The value, or the defaultIntValue if any error occurred.
     */
    public static int getIntValue(String optionName) {
    	
    	if (configOptions.containsKey(optionName)) {
    		
    		String value = configOptions.get(optionName);
    		if (value == null) {
    			Logging.error("ConfigurationParser: No values for option '" + optionName + "'");
    			return ConfigurationParser.defaultIntValue;
    		}
    		else {
    			try {
    				return Integer.parseInt(value);
    			}
    			catch (NumberFormatException e) {
    				Logging.error("ConfigurationParser: Invalid integer value: '" + value + "'");
    				return ConfigurationParser.defaultIntValue;
    			}
    		}
    		
    	}
    	else {
    		Logging.debug("ConfigurationParser: Cannot retrieve unregistered option '" +
    			optionName + "'");
    		return ConfigurationParser.defaultIntValue;
    	}
    	
    }
    
    /**
     * Get the float value for the requested option.
     * 
     * @param optionName The option name.
     * @return The value, or the defaultFloatValue if any error occurred.
     */
    public static float getFloatValue(String optionName) {
    	
    	if (configOptions.containsKey(optionName)) {
    		
    		String value = configOptions.get(optionName);
    		if (value == null) {
    			Logging.error("ConfigurationParser: No values for option '" + optionName + "'");
    			return ConfigurationParser.defaultFloatValue;
    		}
    		else {
    			try {
    				return Float.parseFloat(value);
    			}
    			catch (NumberFormatException e) {
    				Logging.error("ConfigurationParser: Invalid float value: '" + value + "'");
    				return ConfigurationParser.defaultFloatValue;
    			}
    		}
    		
    	}
    	else {
    		Logging.debug("ConfigurationParser: Cannot retrieve unregistered option '" +
    			optionName + "'");
    		return ConfigurationParser.defaultFloatValue;
    	}
    	
    }
    
    /**
     * Get the String value for the requested option.
     * 
     * @param optionName The option name.
     * @return The value, or an new String object if any error occurred.
     */
    public static String getStringValue(String optionName) {
    	
    	if (configOptions.containsKey(optionName)) {
    		
    		String value = configOptions.get(optionName);
    		if (value == null) {
    			Logging.error("ConfigurationParser: No values for option '" + optionName + "'");
    			return new String();
    		}
    		else
    			return value;
    		
    	}
    	else {
    		Logging.debug("ConfigurationParser: Cannot retrieve unregistered option '" +
        			optionName + "'");
    		return new String();
    	}
    	
    }
    
    /**
     * Open a file for configuration parsing. Reads each line, skipping the
     * comments and add the valid values to a static hash of the
     * ConfigurationParser class.
     * 
     * @param filename The filename with it's path.
     * @return <b>true</b> if the file was parsed or <b>false</b> if an
     *         exception occurred.
     */
    public static boolean readFile(String filename) {
    	
    	int				lineNumber 		= 0;
        boolean 		fillElementList = true;
        String 			configLine,
        				optionName,
        				someValue;
        StringTokenizer tokenizedConfigLine;
        
        /*
         * If no option names was registered. Any option name will be
         * accepted (automatically registered).
         */
        if (!configOptions.isEmpty()) fillElementList = false;
        
        try {

        	BufferedReader configFileHandler = new BufferedReader(new FileReader(filename));

        	// Read each line
        	while ((configLine = configFileHandler.readLine()) != null) {

        		lineNumber++;
        		configLine = configLine.trim();
        		
        		// Check if the the line should not be ignored.
        		if (!configLine.startsWith(ConfigurationParser.commentSign) &&
        				!configLine.equals("")) {

        			tokenizedConfigLine = new StringTokenizer(configLine);
        			if (tokenizedConfigLine.hasMoreTokens()) {

        				optionName = tokenizedConfigLine.nextToken();

        				if (configOptions.containsKey(optionName)	|| // The option is registered
        					fillElementList							|| // Accept any option
        					((optionName.endsWith("=") || optionName.endsWith(":")) && // The option has the '=' or ':' sign in the end 
        					 configOptions.containsKey(optionName.substring(0, optionName.length()-1)))) // but it is registered
        					{
        					
        					if (tokenizedConfigLine.hasMoreTokens()) {

        						someValue = tokenizedConfigLine.nextToken();
        						if (someValue.equals("=") || someValue.equals(":")) {

        							if (tokenizedConfigLine.hasMoreTokens()) {

        								someValue = tokenizedConfigLine.nextToken("\n").trim();

        								// Remove everything after the commentSign (if exists)
        								if (someValue.indexOf(ConfigurationParser.commentSign) != -1) {
        									someValue = someValue.substring(0, someValue.indexOf(ConfigurationParser.commentSign));
        								}
        								
        								if (lockedOption.get(optionName)) {
        									Logging.warning("ConfigurationParser: Ignoring option '" + optionName +
                									"'. Using value set from command line.");        									
        								}
        								else {
        									// Finally set the value
        									Logging.trace("ConfigurationParser: Setting '" + optionName + 
        											"' as '" + someValue + "'");
        									configOptions.put(optionName, someValue);
        									lockedOption.put(optionName, false);
        								}

        							}

        						}
        						else if (optionName.endsWith("=") || someValue.endsWith(":")) {

        							// Remove the signal
        							optionName = optionName.substring(0, optionName.length()-1);

        							// Create the StringTokenizer again to get the correct "someValue".
        							tokenizedConfigLine = new StringTokenizer(configLine);
        							tokenizedConfigLine.nextToken();
        							someValue = tokenizedConfigLine.nextToken("\n").trim();

        							// Remove everything after the commentSign (if exists)
        							if (someValue.indexOf(ConfigurationParser.commentSign) != -1) {
        								someValue = someValue.substring(0, someValue.indexOf(ConfigurationParser.commentSign));
        							}

        							if (lockedOption.get(optionName)) {
    									Logging.warning("ConfigurationParser: Ignoring option '" + optionName +
            									"'. Using value set from command line.");        									
    								}
    								else {
    									// Finally set the value
    									Logging.trace("ConfigurationParser: Setting '" + optionName + 
    											"' as '" + someValue + "'");
    									configOptions.put(optionName, someValue);
    									lockedOption.put(optionName, false);
    								}

        						}
        						else {
        							Logging.error("ConfigurationParser: Missing '=' or ':' in option '" + optionName +
        									"' at line " + lineNumber);
        						}
        					}
        					else {
                				Logging.error("ConfigurationParser: Invalid config line '" + configLine +
                						"' at line " + lineNumber);
                			}
        				}
        				else {
        					Logging.error("ConfigurationParser: Unregistered option '" + optionName +
        							"' at line " + lineNumber);
        				}
        			}
        			else {
        				Logging.error("ConfigurationParser: Invalid config line '" + configLine +
        						"' at line " + lineNumber);
        			}
        		}
        	}
        	
        	return true;
        	
        }
        catch (FileNotFoundException e) {
            Logging.error("ConfigurationParser: File not found '" + filename + "'.");
        }
        catch (Exception e) {
            Logging.error("ConfigurationParser: Cannot read configuration file '" +
                filename + "' (" + e + ")");    
        }
        
        return false;
        
    }

}
