/*
 * DataStructureManager.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			19 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.Datastructure;

import br.usp.ime.linux.lemes.ConfigurationParser;
import br.usp.ime.linux.lemes.Logging;

/**
 * The DataStructureManager objects provide methods to manage an internal
 * tree of UrlNode objects that stores porn probabilities for thousands
 * of URLs.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class DataStructureManager {

	// The root UrlNode.
	private UrlNode	rootNode;
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("urlnode_lifetime");
	
	}
	
	/**
	 * Class constructor.
	 */
	public DataStructureManager() {
		
		this.rootNode	= new UrlNode("root");
		this.rootNode.setPornProbability(0);
		
	}
	
	/**
	 * Search for the current URL in the data structure.
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @return The last node reached.
	 */
	private UrlNode search(UrlParser url) {

		UrlNode currentNode = this.rootNode,
				nextNode	= this.rootNode;
		
		url.reset();
		while (url.hasMoreNodes() && nextNode != null) {
			nextNode = currentNode.getLeaf(url.nextNode());
			if (nextNode != null) currentNode = nextNode;
		}
		return currentNode;
		
	}
	
	/**
	 * Check if an URL is stored in the data structure.
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @return <b>true</b> if the URL is in data structure or <b>false</b> if don't.
	 */
	public boolean isStoredUrl(UrlParser url) {
		
		this.search(url);
		
		if (url.hasMoreNodes())
			return false;
		else
			return true;
		
	}
	
	/**
	 * Check if an URL has any pornography info.
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @return <b>true</b> if the URL has some pornography info or <b>false</b> if don't.
	 */
	public boolean hasPornographyInfo(UrlParser url) {
		
		UrlNode search = this.search(url);
		
		if (url.hasMoreNodes())
			return false;
		
		if (search.getPornProbability() == -1)
			return false;
		else
			return true;
		
	}
	
	/**
	 * Check if an URL is expired (Last access exceeds the timeout value in config file).
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @return <b>true</b> if the URL if expired or <b>false</b> if don't.
	 */
	public boolean isExpired(UrlParser url) {
		
		int timeout = ConfigurationParser.getIntValue("urlnode_lifetime");
		UrlNode search = this.rootNode;
		String nodeName;
		
		url.reset();
		while (url.hasMoreNodes()) {
			nodeName = url.nextNode();
			search = search.getLeaf(nodeName);
			if (search == null) return true;
		}
		if ((search.getCreateDate().getTimeInMillis() + (timeout * 1000)) < System.currentTimeMillis())
			return true;
		else
			return false;
		
	}
	
	/**
	 * Add info about an URL to the data structure.
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @param pornProbability The porn probability assigned to the URL (float
	 *        value between 0 and 1)
	 */
	public void addUrlInfo(UrlParser url, float pornProbability) {
		

		UrlNode	search		= this.rootNode,
				nextNode	= this.rootNode;
		String 	nodeName	= "root",
				label		= "";
		
		Logging.trace(this, "Adding URL info for: " + url.toString());
		
		url.reset();
		while (url.hasMoreNodes() && nextNode != null) {
			nodeName	= url.nextNode();
			
			if (label.equals("")) label	= nodeName;
			else label = label.concat("_" + nodeName);
			
			nextNode	= search.getLeaf(nodeName);
			
			if (nextNode != null) search = nextNode;
		}
		
		if (nextNode == null) {
			
			search = search.addLeaf(nodeName, label);
			
			while (url.hasMoreNodes()) {
				nodeName	= url.nextNode();
				label	= label.concat("_" + nodeName);
				search		= search.addLeaf(nodeName, label);
			}
		}
		
		Logging.trace(this, "Setting porn probability as " + pornProbability);
		search.setPornProbability(pornProbability);
		
	}
	
	/**
	 * Get the porn probability for an URL.
	 * 
	 * @param url The URL represented by an UrlParser object.
	 * @return The porn probability for the URL.
	 */
	public float getObjectPornProbability(UrlParser url) {
		
		UrlNode search = this.search(url);
		
		// If the object could not be reached we return 0.
		if (url.hasMoreNodes())
			return 0;
		else
			return search.getPornProbability();
		
	}
	
}
