/*
 * UrlNode.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			03 de janeiro de 2008
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.Datastructure;

import java.io.*;
import java.util.*;
import br.usp.ime.linux.lemes.Logging;
import br.usp.ime.linux.lemes.ConfigurationParser;

/**
 * This class represents each part of an URL. The URL is divided in several
 * nodes that may have common parts to other URLs, and then we have a tree
 * view that can group similar URLs.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1.1
 */
public class UrlNode {

	private Calendar 					createDate;
	private Calendar					lastAccess;
	private String 						nodeName;
	private String						swapFilename;
	private Vector<String>				leafsNames;
	private HashMap<String, UrlNode>	leafs;
	private float						pornProbability;
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("datastructure_swap_filepath");
		
	}
	
	/**
	 * Class constructor. Requires a name to identify the node in the tree.
	 * @param name The node name.
	 */
	public UrlNode(String name) {
		
		// Time stamps
		createDate 				= Calendar.getInstance();
		lastAccess 				= createDate;
		
		// Node essential info
		this.nodeName 			= name;
		this.pornProbability 	= -1;
		this.swapFilename		= null;
		
		// These objects are only created if they are really needed
		this.leafsNames 		= null;
        this.leafs              = null;
        
	}
	
	/**
	 * Class destructor. Removes the swap file, if exists, of the current object.
	 */
	public void finalize() {
		
		if (this.swapFilename != null) {
			File tempFile = new File(this.swapFilename);
			tempFile.delete();
		}
		
	}
	
	/**
	 *  Update the last access info of the current object.
	 */
	private void refreshAccess() {
		
		this.lastAccess	= Calendar.getInstance();
		
	}
	
	/**
	 * Check if this node has leafs.
	 * 
	 * @return <b>true</b> if the node has leafs, <b>false</b> if don't.
	 */
	private boolean hasLeafs() {
		
		if (this.leafs == null && this.swapFilename == null)
			return false;
		else
			return true;
		
	}
	
	/**
	 * Check for the leafs structure and swap if from file if needed.
	 */
	private void checkLeafs() {
		
		/* Initialize some objects if needed */
		if (this.leafs == null) {
			if (this.swapFilename == null) {
				Logging.trace(this, "The current node [" + this.nodeName +
						"] has no leafs (null). Initiating leafs structure.");
				this.leafsNames = new Vector<String>();
				this.leafs      = new HashMap<String, UrlNode>();
			}
			else {
				/* Need to retrieve data from file. Create the objects on errors */						
				if (!this.swapFromFile()) {
					Logging.failure(this, "Swapping in failed for node '" + this.nodeName + 
							"' using file " + this.swapFilename + ".");
					this.leafsNames = new Vector<String>();
					this.leafs      = new HashMap<String, UrlNode>();
				}
				else
					Logging.trace(this, "Swapping in node '" + this.nodeName + "' done!");
			}
		}
		
	}
	
	/**
	 * Return the UrlNode object, leaf of the current one, that has some name
	 * assigned.
	 * 
	 * @param name The name assigned to the UrlNode.
	 * @return The UrlNode object or <b>null</b> if there is no node with
	 *         this name.
	 */
	public UrlNode getLeaf(String name) {
		
		this.refreshAccess();
		
		// Check leafs structure
		this.checkLeafs();
		
		// Check if there is any branch with this node name
		if (this.leafs.containsKey(name)) {
			return this.leafs.get(name);
		}
		else {
			return null;
		}
		
	}
	
	/**
	 * Create and return a new UrlNode and assign it with a name to the
	 * current UrlNode.
	 * 
	 * @param name The name assigned to the new node.
	 * @param id The id of the new UrlNode.
	 * @return The new UrlNode.
	 */
	public UrlNode addLeaf(String name, String id) {
		
		this.refreshAccess();
		
		// Check leafs structure
		this.checkLeafs();
		
		// Add new leaf
		UrlNode newNode = new UrlNode(id);
		this.leafsNames.add(name);
		this.leafs.put(name, newNode);
		
		return newNode;
		
	}
	
	/**
	 * Getter for the internal Calendar that stores the create date of the
	 * current object.
	 * 
	 * @return The Calendar object.
	 */
	public Calendar getCreateDate() {
		
		return this.createDate;
		
	}
	
	/**
	 * Getter for the internal Calendar that stores the last access date.
	 * 
	 * @return The Calendar object.
	 */
	public Calendar getLastAccess() {
		
		return this.lastAccess;
		
	}
	
	/**
	 * Set a new value for the porn probability of the current node.
	 * 
	 * @param pornProbability The porn probability (float value between 0
	 *        and 1).
	 */
	public void setPornProbability(float pornProbability) {
		
		this.pornProbability = pornProbability;
		
	}
	
	/**
	 * Get the porn probability for the current node.
	 * 
	 * @return The porn probability (float value between 0 and 1).
	 */
	public float getPornProbability() {
		
		if (this.hasLeafs()) {
						
			this.checkLeafs();
			
			/*
			 * Now we need to create a probability to this branch.
			 * Lets return the porn average of all nodes.
			 */ 
			int		index;
			float 	porn	= 0, 
					count	= 0;
			
			// The current node (if defined)
			if (this.pornProbability != -1) {
				porn = this.pornProbability;
				count++;
			}
			
			// The leafs probability
			for (index = 0; index < this.leafsNames.size(); index++) {
				porn += this.leafs.get(this.leafsNames.elementAt(index)).getPornProbability();
				count++;
			}
			
			return (porn / count);
			
		}
		else {
			if (this.pornProbability == -1)
				return 0;
			else
				return this.pornProbability;
		}
		
	}
	
	/**
	 * Swap the content of this node and its leafs to disk. The destination
	 * folder is defined for the ConfigurationParser class using the setting
	 * named 'datastructure_swap_filepath'.
	 * 
	 * @return <b>true</b> if the swapping was successful or <b>false</b> if don't. 
	 */
	public boolean swapToFile() {
		
		// Get path to write swap files
		String swapFilepath = ConfigurationParser.getStringValue("datastructure_swap_filepath");
		if (!swapFilepath.equals("") && !swapFilepath.endsWith("/"))
			swapFilepath = swapFilepath.concat("/");
		this.swapFilename = swapFilepath + this.nodeName + "_"+ System.currentTimeMillis();
		
		Logging.debug(this, "Starting swapToFile() procedure for node " + this.nodeName);
		try {
			
			Logging.debug(this, "Writing to file: " + this.swapFilename);
			PrintWriter swapFile = new PrintWriter(new FileWriter(this.swapFilename));
			this.toFile(swapFile);
			swapFile.close();
			
			// Flush this info to retrieve space
			this.leafs 		= null;
			this.leafsNames = null;
			
			return true;
			
		}
		catch (IOException e) {
			Logging.failure(this, "Cannot write swapfile at '" + 
				this.swapFilename + "'");
			return false;
		}
	}
	
	/**
	 * Swap the content back of this node from file to memory.
	 * 
	 * @return <b>true</b> if the swapping was successful or <b>false</b> if don't.
	 */
	public boolean swapFromFile() {
		
		if (this.swapFilename == null) {
			Logging.error(this, "Cannot swapFromFile() when swapFilename is null.");
			return false;
		}
		
		Logging.debug(this, "Swapping data from file " + this.swapFilename);
		
		try {
			BufferedReader inputFile = new BufferedReader(new FileReader(this.swapFilename));
			this.fromFile(inputFile);
			inputFile.close();
			File tempFile = new File(this.swapFilename);
			tempFile.delete();
			this.swapFilename = null;
			return true;
		}
		catch (IOException exception){
			Logging.failure(this, "Cannot open swapfile at " + this.swapFilename +
					" (" + exception + ")");
			Logging.debug(this, "Lost node and subtree info. Creating new structure.");
			return false;
		}
	}
	
	/**
	 * Display the UrlNodes recursively in an idented way.
	 * 
	 * @param spacer A String that carries the "offset" to display this node.
	 */
	public void display(String spacer) {
		
		Logging.trace(this, "[display]" + spacer + nodeName);
		
		if (this.leafs == null && this.swapFilename != null)
			this.swapFromFile();
		
		if (leafsNames != null) {
			
			spacer = spacer.concat("  ");
		
			int index;
			for (index = 0; index < leafsNames.size(); index++) {
				leafs.get(leafsNames.get(index)).display(spacer);
			}
			
		}
		
	}
	
	/**
	 * Save the UrlNode and it's leafs to an PrintWriter object.
	 * 
	 * @param outputFile The PrintWriter object.
	 */
	public void toFile(PrintWriter outputFile) {
		
		if (this.leafs == null && this.swapFilename != null) {
			
			Logging.debug(this, "toFile reading data from file " + this.swapFilename);
			
			try {
				BufferedReader inputFile = new BufferedReader(new FileReader(this.swapFilename));
				String temp;
				while ((temp = inputFile.readLine()) != null)
					outputFile.print(temp);
				inputFile.close();				
			}
			catch (IOException e){
				Logging.failure(this, "Cannot open swapfile at " + this.swapFilename +
						" (" + e + ")");
				Logging.debug(this, "Lost node and subtree info. Creating new structure.");
				outputFile.print("(" + this.nodeName + "\t" + this.pornProbability + "\t)");	
			}
			
		}
		else {
			
			int index;
		
			// Verbose
			Logging.trace(this, "Writing node " + this.nodeName);
		
			// This node info
			outputFile.print("(" + this.nodeName + "\t" + this.pornProbability + "\t");
				
			// Leafs info
			if (leafsNames != null) {
				for (index = 0; index < leafsNames.size(); index++) {
					leafs.get(leafsNames.get(index)).toFile(outputFile);
					if (index < leafsNames.size() - 1) outputFile.print(",");
				}
			}
		
			// End of info
			outputFile.print(")");
		}
	}
	
	/**
	 * Read the UrlNode from file, and it's leafs if exists, from
	 * a BufferedReader object.
	 * 
	 * @param inputFile The BufferedReader object.
	 * @return <b>true</b> if it was successful or <b>false</b> if don't.
	 */
	public boolean fromFile(BufferedReader inputFile) {
		
		char[]	charBuffer = new char[1];
		String	miniBuffer,
				localBuffer;
		
		UrlNode tempUrlNode;
						
		try {
			// Check if there is a starting parentesis
			inputFile.mark(2);
			inputFile.read(charBuffer, 0, 1);
			miniBuffer = new String(charBuffer);
			if (!miniBuffer.equals("(")) inputFile.reset();

			// Retrieve nodename from file
			this.nodeName = "";
			inputFile.read(charBuffer, 0, 1);
			miniBuffer = new String(charBuffer);
			while (!miniBuffer.equals("\t")) {
				this.nodeName = this.nodeName.concat(miniBuffer);
					
				inputFile.read(charBuffer, 0, 1);
				miniBuffer = new String(charBuffer);
				
			}
			Logging.trace(this, "Defining node name as '" + this.nodeName + "'");
		
			// Set porn probability
			localBuffer = "";
			inputFile.read(charBuffer, 0, 1);
			miniBuffer = new String(charBuffer);
			while (!miniBuffer.equals("\t")) {
				localBuffer = localBuffer.concat(miniBuffer);
				inputFile.read(charBuffer, 0, 1);
				miniBuffer = new String(charBuffer);
			}
			this.pornProbability = Float.parseFloat(localBuffer);
			
			// If there are leafs above, create leaf and call the method recursively
			inputFile.mark(2);
			inputFile.read(charBuffer, 0, 1);
			miniBuffer = new String(charBuffer);
						
			while (!miniBuffer.equals(")")) {
				inputFile.reset();
				
				// Check if there is a comma separating multiple leafs
				inputFile.read(charBuffer, 0, 1);
				miniBuffer = new String(charBuffer);
				if (!miniBuffer.equals(",")) inputFile.reset();
		
				// Initialize some objects if needed
				if (this.leafs == null) {
					this.leafsNames = new Vector<String>();
					this.leafs      = new HashMap<String, UrlNode>();
				}
				
				// Add new leaf/branch
				Logging.trace(this, "Creating undefined subnode from file.");
				tempUrlNode = new UrlNode("[UNDEF]");
				tempUrlNode.fromFile(inputFile);
				
				localBuffer = tempUrlNode.nodeName;
				leafsNames.add(localBuffer);
				leafs.put(localBuffer, tempUrlNode);			
				
				inputFile.mark(2);
				inputFile.read(charBuffer, 0, 1);
				miniBuffer = new String(charBuffer);
			}
		
			return true;
		}
		catch (NumberFormatException e) {
			Logging.error(this, "Invalid number format for node porn probability (" + e + ")");
		}
		catch (IOException e) {
			Logging.error(this, "Error reading file data. Aborting.");
		}
		
		return false;
		
	}
	
}
