/*
 * UrlParser.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			25 de julho de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.Datastructure;

import java.util.*;
import br.usp.ime.linux.lemes.Logging;

/**
 * This class is responsible to receive an url as a string and retrieve some
 * informations, like file extension. The main goal of this class is tokenize
 * the url, creating nodes that are fetched to build a tree data structure,
 * according to the SquidPCB project specifications.
 * <p>
 * A url should be something like:<p>
 * [http://]www.somesite.com[:portnumber]/[somefolder/][somefile][.extension]
 * [?getarguments]
 * <p>
 * (all elements in brackets are optional)
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1a
 */
public class UrlParser {
	
	private String		originalUrl;
	private String[] 	itemList;
	private String		fileExtension;
	private int			itemIndex, itemCount;
	
	/**
	 * Class constructor. Requires a string containing the url of the object.
	 */
	public UrlParser(String providedUrl) {
		
		Vector<String>	localItemNames 	= new Vector<String>();
		Stack<String> 	reverseHostname	= new Stack<String>();
		
		// Save the URL provided for future use
		this.originalUrl 	= providedUrl;
		this.fileExtension 	= new String("");
		Logging.trace(this, "Original URL: '" + this.originalUrl + "'");
		
		// Remove any protocol info
		int cuttingPoint = providedUrl.indexOf("://");
		if (cuttingPoint != -1)
			providedUrl = providedUrl.substring(cuttingPoint + 3);

		// Log activity
		Logging.trace(this, "Parsing URL: '" + providedUrl + "'");
		
		// Split host for filepath
		StringTokenizer tokenizedUrl = new StringTokenizer(providedUrl);
		String urlHost = tokenizedUrl.nextToken("/");
		
		// Remove port number
		cuttingPoint = urlHost.indexOf(":");
		if (cuttingPoint != -1)
			urlHost = urlHost.substring(0, cuttingPoint);
		
		// Push all parts in the stack to put them in the vector in reverse order
		StringTokenizer tokenizedHostPath = new StringTokenizer(urlHost);
		while (tokenizedHostPath.hasMoreTokens())
			reverseHostname.push(tokenizedHostPath.nextToken("."));	
		while (!reverseHostname.empty())
			localItemNames.addElement(reverseHostname.pop());
		
		if (tokenizedUrl.hasMoreTokens()) {
			// And finally the file path
			String filePath = tokenizedUrl.nextToken("?");
			if (filePath.length() > 0) { 
				StringTokenizer tokenizedFilePath = new StringTokenizer(filePath, "/");
				String filePathPart = new String();
				while (tokenizedFilePath.hasMoreTokens()) {
					// Add the token to the vector
					filePathPart = tokenizedFilePath.nextToken();
					if (filePathPart.length() > 0)
						localItemNames.addElement(filePathPart);
				}
				// Check if there is a file extension
				cuttingPoint = filePathPart.lastIndexOf(".");
				if (cuttingPoint != -1) {
					this.fileExtension = filePathPart.substring(cuttingPoint + 1);
					Logging.trace(this, "File extension is '" + this.fileExtension + "'.");
				}
			}
			else {
				localItemNames.addElement("%undefined_file_path%");
			}
		}
		
		// And finally the GET arguments if found
		while (tokenizedUrl.hasMoreTokens()) {
			localItemNames.addElement(tokenizedUrl.nextToken());
		}
					
		// Time to set all object variables
		this.itemCount 	= localItemNames.size();
		this.itemList  	= new String[this.itemCount];
		for (itemIndex = 0; itemIndex < this.itemCount; itemIndex++) {
			this.itemList[itemIndex] = localItemNames.elementAt(itemIndex);
		}
		
		// Reset pointer
		this.itemIndex 	= 0;
		
	}
	
	/**
	 * Reset the internal pointer to next node.
	 */
	public void reset() {
		
		this.itemIndex = 0;
		
	}
	
	/**
	 * Fetch the next node from the url.
	 *
	 * @return The next node from URL.
	 */
	public String nextNode() {
		
		if (this.itemIndex == this.itemCount) {
			Logging.error(this, "Trying to access the next node after end of nodes.");
			return "";
		}
		else return this.itemList[this.itemIndex++];
		
	}
	
	/**
	 * Check if there are more nodes to fetch.
	 *  
	 * @return <b>true</b> if there are mode nodes to fetch or <b>false</b> if don't. 
	 */
	public boolean hasMoreNodes() {
		
		if (this.itemIndex < this.itemCount) return true;
		else return false;
		
	}
	
	/**
	 * Return the original string used to create the object.
	 * 
	 * @return The url string.
	 */
	public String getUrl() {
		
		return this.originalUrl;
		
	}
	
	/**
	 * Return the parsed file extension from the file on url.
	 * 
	 * @return The file extension (without dots).
	 */
	public String getFileExtension() {
		
		return this.fileExtension;
		
	}
	
	/**
	 * Retrieve a representation of this object with the nodes delimited by
	 * brackets.
	 * 
	 * @return The string representation.
	 */
	public String toString() {
		
		int 	index;
		String 	returnValue = "";
		
		// Build string
		for (index = 0; index < this.itemCount; index++) {
			returnValue = returnValue.concat("[" + this.itemList[index] + "]");
		}
		
		return returnValue;
		
	}
	
}
