/*
 * ConnectionClient.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			26 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.IPC;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.StringTokenizer;
import br.usp.ime.linux.lemes.Logging;

/**
 * This class is responsible to connect and authenticate to a ConnectionThread
 * object to perform the requests INFO and LIST in the remote data structure.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class ConnectionClient {

	private Socket			localSocket;
	private BufferedReader	fromSocket;
	private PrintStream		toSocket;
	
	private boolean			isConnected = false;
	private float			pObj = 0,
							pGrp = 0,
							pMax = 0,
							pAvg = 0;
	
	/**
	 * A class constructor that doesn't connect to any server.
	 */
	public ConnectionClient() {
		
		// A "Do nothing" constructor
		
	}
	
	/**
	 * Create a ConnectionClient object. This constructor starts a TCP connection to server and
	 * do all the authentication process, according to the SquidPCB protocol.
	 * 
	 * @param ip The server IP address.
	 * @param port The TCP port to connect.
	 * @param username The username to authenticate.
	 * @param password The password of this username.
	 */
	public ConnectionClient(InetAddress ip, int port, String username, String password) {
		
		try {
			
			// Open network connection
			localSocket			= new Socket(ip, port);
			this.fromSocket		= new BufferedReader(new InputStreamReader(this.localSocket.getInputStream()));
			this.toSocket		= new PrintStream(this.localSocket.getOutputStream());
			
			// Start of authentication
			String response = this.fromSocket.readLine();
			
			if (response.startsWith("SquidPCB")) {
				
				// Send the username
				this.toSocket.println("USER " + username);
				response = this.fromSocket.readLine();
				if (response.startsWith("200 ")) {
					
					// Send the password
					this.toSocket.println("PASS " + password);
					response = this.fromSocket.readLine();
					if (response.startsWith("200 ")) {
						
						// Connection to SquidPCB is ready
						this.isConnected = true;
						Logging.debug(this, "Connection authenticated. Ready to proccess requests.");
						
					}
					else if (response.startsWith("410 ")) {
						Logging.error(this, "Server rejected password. Please check it.");
					}
					else {
						Logging.panic(this, "Unknown error sending 'PASS' command.");
					}
					
				}
				else if (response.startsWith("410 ")) {
					Logging.error(this, "Server rejected username '" + username + "'.");
				}
				else {
					Logging.panic(this, "Unknown error sending 'USER' command.");
				}
				
			}
			else {
				Logging.panic(this, "Could not identify server protocol.");
			}
			
		}
		catch (Exception e) {
			Logging.panic(this, "Could not connect to server. (" + e + ")");
		}
		
	}
	
	/**
	 * Check if the current connection is authenticated.
	 * 
	 * @return <b>true</b> if is authenticated or <b>false</b> if don't.
	 */
	public boolean isConnected() {
		
		return this.isConnected;
		
	}
	
	/**
	 * Get INFO (see the connection protocol) of an URL. The result must be retrieved using
	 * the getObj(), getGrp(), getMax() and getAvg() methods.
	 * 
	 * @param url The URL of the object.
	 */
	public boolean getInfo(String url) {
		
		// Check if the connection was sucessfull authenticated
		if (!this.isConnected) {
			Logging.trace(this, "Trying to use the INFO command in a non authenticated connection.");
			return false;
		}
		
		// Reset the probabilities
		this.pObj = 0;
		this.pGrp = 0;
		this.pMax = 0;
		this.pAvg = 0;
		
		// Do the query
		this.toSocket.println("INFO " + url);
		try {
			
			// Get the response and try to parse its tokens
			String	response = this.fromSocket.readLine(),
					part, info;
			float	value;
			
			if (response.startsWith("200 ")) {
			
			StringTokenizer tokenizedResponse = new StringTokenizer(response.substring(4)),
							tokenizedPart;
			boolean continueParsing = true;
			
			while (tokenizedResponse.hasMoreTokens() && continueParsing) {
				
				part			= tokenizedResponse.nextToken();
				tokenizedPart	= new StringTokenizer(part, "=");
				
				if (tokenizedPart.hasMoreTokens())
					info = tokenizedPart.nextToken();
				else
					info = new String("");
				
				if (tokenizedPart.hasMoreTokens()) {
					try {
						value = Float.parseFloat(tokenizedPart.nextToken());
					}
					catch (Exception e) {
						Logging.warning(this, "Unrecognizable returned value for 'INFO' command: " + part);
						value = 0;
					}
				}
				else
					value = 0;
				
				// Check the argument
				if (info.equalsIgnoreCase("OBJ")) {
					this.pObj = value; 
				}
				else if (info.equalsIgnoreCase("GRP")) {
					this.pGrp = value;
				}
				else if (info.equalsIgnoreCase("MAX")) {
					this.pMax = value;
				}
				else if (info.equalsIgnoreCase("AVG")) {
					this.pAvg = value;
				}
				else if (info.startsWith("(")) {
					continueParsing = false;
				}
				else {
					Logging.warning(this, "Unrecognizable returned probability for 'INFO' command: " + info);
				}
				
			}
			
			// Done!
			return true;
			
			}
			else
				Logging.debug(this, "Error while performing 'INFO' command with URL: '" +
						url + "' (4XX return message)");
			
		}
		catch (Exception e) {
			Logging.debug(this, "Error while performing 'INFO' command with URL: '" +
					url + "' (" + e + ")");
			this.isConnected = false;
		}
		
		return false;
		
	}
	
	/**
	 * Close the connection to server.
	 */
	public void close() {
		try {
			
			// Try to be "polite"
			this.toSocket.println("QUIT");
			this.fromSocket.readLine(); // The "Bye." response.
			
			// Close pipes and socket
			this.fromSocket.close();
			this.toSocket.close();
			this.localSocket.close();
			
		}
		catch (Exception e) {
			Logging.debug(this, "Exception caught when closing ConnectionClient (" + e + ")");
		}
	}
	
	/**
	 * Getter for object porn probability.
	 * @return The porn probability between 0 and 1.
	 */
	public float getPObj() {
		
		return this.pObj;
		
	}
	
	/**
	 * Getter for group porn probability (object at the same level).
	 * @return The porn probability between 0 and 1.
	 */
	public float getPGrp() {
		
		return this.pGrp;
		
	}
	
	/**
	 * Getter for average porn probability in the url path.
	 * @return The porn probability between 0 and 1.
	 */
	public float getPAvg() {
		
		return this.pAvg;
		
	}
		
	/**
	 * Getter for the maximum porn probability in the url path.
	 * @return The porn probability between 0 and 1.
	 */
	public float getPMax() {
		
		return this.pMax;
		
	}
	
}
