/*
 * ConnectionServer.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			21 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.IPC;

import java.net.*;
import br.usp.ime.linux.lemes.ConfigurationParser;
import br.usp.ime.linux.lemes.Logging;
import br.usp.ime.linux.lemes.Datastructure.DataStructureManager;
import br.usp.ime.linux.lemes.WordSearch.WordSearch;

/**
 * The ConnectionServer class creates a TCP socket listener that wait for
 * connections and instantiate ConnectionThread objects to handle all the
 * client requests.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 * 
 * TODO: Create a pool of ConnectionThreads.
 */
public class ConnectionServer {

	// Some objects that must be used for every ConnectionThread
	private DataStructureManager	dsManager;
	private WordSearch				urlWordSearch,
									textWordSearch;
	private boolean					running;
	
	/**
	 * Class constructor. Requires some objects that are required for the
	 * ConnectionThread objects and must be common to all threads.
	 * 
	 * @param dsManager The DataStructureManager object that maintains all
	 *        porn probabilities for each URL.
	 * @param urlWordSearch The object that perform searchs for words in the
	 *        URL string.
	 * @param textWordSearch The object that perform searchs for words in the
	 *        content of the retrieved text objects.
	 */
	public ConnectionServer(DataStructureManager dsManager, WordSearch urlWordSearch, WordSearch textWordSearch) {
		
		this.dsManager		= dsManager;
		this.urlWordSearch	= urlWordSearch;
		this.textWordSearch	= textWordSearch;
		
	}
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("server_portnumber", "7722");
		ConfigurationParser.setOptionName("bind_interface_ip");
	
	} 
	
	/**
	 * Start the TCP server. Listen for incomming connections that are treated by
	 * ConnectionThread objects.
	 */
	public void start() {
		
		int				port;
		InetAddress		bindIp = null;
		ServerSocket	myServerSocket;
		
		// Get the port number to listen.
		port = ConfigurationParser.getIntValue("server_portnumber");
		if (port < 1 || port > 65535) {
			Logging.panic(this, "Invalid server port number: " + port + ". Aborting.");
		}
		
		// Check if there is some specific interface address to listen.
		if (ConfigurationParser.isDefined("bind_interface_ip")) {
			try {
				bindIp = InetAddress.getByName(ConfigurationParser.getStringValue("bind_interface_ip"));
			}
			catch (Exception e)  {
				Logging.panic(this, "Invalid ip address: " + 
						ConfigurationParser.getStringValue("bind_interface_ip") + "'. (" + e + ")");
			}
		}
		
		// Try to open socket
		try {
			
			if (bindIp != null) {
				myServerSocket	= new ServerSocket(port, 0, bindIp);
			}
			else {
				myServerSocket = new ServerSocket(port);
			}
			
			/*
			 * TODO: Treat signals to make somethings like:
			 * - Shutdown
			 * - Restart / Reload configs
			 */
			this.running		= true;
			Socket				thisSocket;
			ConnectionThread	someConnectionThread;
									
			while (this.running) {
				try {
					
					// Get a connection
					thisSocket				= myServerSocket.accept();
					
					// Start a thread to the connection
					someConnectionThread	= new ConnectionThread(this, thisSocket);
					someConnectionThread.start();
					
				}
				catch (Exception e) {
					Logging.failure(this, "Error while accepting new connection. (" + e + ")");
				}
			}
			
			
		}
		catch (Exception e) {
			Logging.panic(this, "Cannot open TCP port #" + port +
					" for incomming connections. (" + e + ")");
		}
		
	}

	/**
	 * Shutdown this server.
	 */
	public void shutdown() {
		this.running = false;
	}
	
	/**
	 * Getter for the DataStructureManager that must be used for all ConnectionThreads.
	 * 
	 * @return The instance's DataStructureManager object.
	 */
	public DataStructureManager getDsManager() {
		
		return this.dsManager;
		
	}

	/**
	 * Getter for the WordSearch object that can be used for more than one ConnectionThread.
	 * 
	 * @return The instance's WordSearch object.
	 */
	public WordSearch getUrlWordSearch() {
		
		return this.urlWordSearch;
		
	}
	
	/**
	 * Getter for other WordSearch object that can be used for more than one ConnectionThread.
	 * 
	 * @return The instance's WordSearch object.
	 */
	public WordSearch getTextWordSearch() {
		
		return this.textWordSearch;
		
	}
	
}
