/*
 * ConnectionThread.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			21 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.IPC;

import java.io.*;
import java.net.Socket;
import java.util.StringTokenizer;
import br.usp.ime.linux.lemes.ConfigurationParser;
import br.usp.ime.linux.lemes.Logging;
import br.usp.ime.linux.lemes.Datastructure.DataStructureManager;
import br.usp.ime.linux.lemes.Datastructure.UrlParser;
import br.usp.ime.linux.lemes.UrlObjectAnalyzer.UrlObjectAnalyzer;
import br.usp.ime.linux.lemes.WordSearch.WordSearch;

/**
 * This class receives a socket object from it's server and takes care of the
 * user authentication and it's requests processing.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class ConnectionThread extends Thread {

	private DataStructureManager	dsManager;
	private WordSearch				urlWordSearch;
	private UrlObjectAnalyzer		localAnalyzer;
	
	private Socket					mySocket;
	private BufferedReader			fromSocket;
	private PrintStream				toSocket;
	private String					username;
	
	private static int				getUsernamePhase	= 0;
	private static int				getPasswordPhase	= 1;
	private static int				acceptRequests		= 2;
	private static int				quitConnection		= 3;
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("limit_url_points");
			
	} 
	
	/**
	 * Class constructor. Receives a reference to the server and the socket
	 * to perform network IO.
	 * 
	 * @param server A reference to the ConnectionServer object.
	 * @param localSocket The socket of the current connection.
	 */
	public ConnectionThread(ConnectionServer server, Socket localSocket) {
		
		// Get some objects from the server
		this.dsManager			= server.getDsManager();
		this.urlWordSearch		= server.getUrlWordSearch();
		this.localAnalyzer		= new UrlObjectAnalyzer(server.getTextWordSearch());
		
		// Set the local socket object
		this.mySocket			= localSocket;
		
		// Get the socket pipes
		try {
			this.fromSocket		= new BufferedReader(new InputStreamReader(localSocket.getInputStream()));
			this.toSocket		= new PrintStream(localSocket.getOutputStream());
		}
		catch (IOException e) {
			Logging.failure(this, "Could not create socket pipes.");
			this.fromSocket		= null;
			this.toSocket		= null;
		}
		
	}

	/**
	 * Run the thread.
	 */
	public void run() {
		
		// Check for some required objects. Maybe some error ocurred when the
		// object was constructed.
		if (this.fromSocket == null || this.toSocket == null)
			Logging.failure(this, "Cannot start ConnectionThread.");
		
		else {
			
			// Welcome message
			this.toSocket.println("SquidPCB v0.1 - Copyright (C) 2007, 2008 Fernando Lemes da Silva. (http://squidpcb.sf.net)");
			
			int currentPhase = ConnectionThread.getUsernamePhase;
			String	commandLine;
						
			try {
				
				while (currentPhase != ConnectionThread.quitConnection) {
				
					commandLine				= this.fromSocket.readLine();
								
					if (currentPhase == ConnectionThread.getUsernamePhase) {
						currentPhase = this.getUsername(commandLine);
					}
					else if (currentPhase == ConnectionThread.getPasswordPhase) {
						currentPhase = this.getPassword(commandLine);
					}
					else if (currentPhase == ConnectionThread.acceptRequests) {
						currentPhase = this.acceptRequest(commandLine);
					}
				}
				
				this.toSocket.println("200 Bye.");
				
			}
			catch (IOException e) {
				Logging.failure(this, "Network failure. Socket closed?");
			}
			
			// Close all pipes and socket.
			try {
				this.fromSocket.close();
				this.toSocket.close();
				this.mySocket.close();
			}
			catch (Exception e) {}
			
		}
	}
	
	/**
	 * Check for a command line and try to parse the USER command.
	 * 
	 * @param commandLine The command line from the socket.
	 * @return The next phase of the connection process.
	 */
	private int getUsername(String commandLine) {
		
		// Quit connection if commandLine is null (probably closed socket)
		if (commandLine == null) return ConnectionThread.quitConnection;
		
		String firstToken, secondToken;
		StringTokenizer tokenizedCommandLine = new StringTokenizer(commandLine);
		
		if (tokenizedCommandLine.hasMoreTokens()) {
			firstToken = tokenizedCommandLine.nextToken();
			
			// Check the quit command
			if (firstToken.equalsIgnoreCase("quit")) {
				return ConnectionThread.quitConnection;
			}
			
			// Check the user command
			if (firstToken.equalsIgnoreCase("user")) {
				if (tokenizedCommandLine.hasMoreTokens()) {
					secondToken = tokenizedCommandLine.nextToken();
					
					// Check if username exists
					if (UserAuthenticator.isValidUser(secondToken)) {
						this.toSocket.println("200 Accepted user '" + secondToken + "'. Go ahead.");
						
						// Set username and proceed to next phase
						this.username = secondToken;
						return ConnectionThread.getPasswordPhase;
						
					}									
					else
						this.toSocket.println("410 Invalid username '" + secondToken + "'.");
				}
				else
					this.toSocket.println("400 Missing username in 'USER' command.");
			}
			
			// Check for other commands that are not allowed at the current phase.
			else if (firstToken.equalsIgnoreCase("pass")) {
				this.toSocket.println("402 You must first use the 'USER' command.");
			}
			else if (firstToken.equalsIgnoreCase("info")) {
				this.toSocket.println("405 You must be authenticated to use this command.");
			}
			else if (firstToken.equalsIgnoreCase("list")) {
				this.toSocket.println("405 You must be authenticated to use this command.");
			}
			
			// Invalid command.
			else
				this.toSocket.println("400 Invalid command.");
		}
		else
			this.toSocket.println("400 Invalid command.");
		
		// Keep the current phase
		return ConnectionThread.getUsernamePhase;
		
	}
		
	/**
	 * Check for a command line and try to parse the PASS command.
	 * @param commandLine The command line from the socket.
	 * @return The next phase of the connection process.
	 */
	private int getPassword(String commandLine) {
		
		// TODO: Improve security and maybe rewrite all the code in a simplest way.
		
		// Quit connection if commandLine is null (probably closed socket)
		if (commandLine == null) return ConnectionThread.quitConnection;
		
		String firstToken, secondToken;
		StringTokenizer tokenizedCommandLine = new StringTokenizer(commandLine);
		
		if (tokenizedCommandLine.hasMoreTokens()) {
			firstToken = tokenizedCommandLine.nextToken();
			
			// Check the quit command
			if (firstToken.equalsIgnoreCase("quit")) {
				return ConnectionThread.quitConnection;
			}
			
			// Check the pass command
			if (firstToken.equalsIgnoreCase("pass")) {
				if (tokenizedCommandLine.hasMoreTokens()) {
					secondToken = tokenizedCommandLine.nextToken();
					
					// Check if password matchs
					if (UserAuthenticator.checkPassword(this.username, secondToken)) {
						this.toSocket.println("200 Accepted password for user '" + this.username + "'.");
						
						// Proceed to next phase
						return ConnectionThread.acceptRequests;
						
					}									
					else {
						try {
							// Sleep 3 seconds. To prevent brute force attack.
							Thread.sleep(3000);
						}
						catch (Exception e) { }
						this.toSocket.println("411 Invalid password.");
					}
				}
				else
					this.toSocket.println("400 Missing password in 'PASS' command.");
			}
			
			// Check for other commands that are not allowed at the current phase.
			else if (firstToken.equalsIgnoreCase("user")) {
				this.toSocket.println("402 The current user is already set.");
			}
			else if (firstToken.equalsIgnoreCase("info")) {
				this.toSocket.println("405 You must be authenticated to use this command.");
			}
			else if (firstToken.equalsIgnoreCase("list")) {
				this.toSocket.println("405 You must be authenticated to use this command.");
			}
			
			// Invalid command.
			else
				this.toSocket.println("400 Invalid command.");
		}
		else
			this.toSocket.println("400 Invalid command.");
		
		// Keep the current phase
		return ConnectionThread.getPasswordPhase;
		
	}
	
	/**
	 * Parse a command line to accept requests for INFO and LIST commands.
	 * 
	 * @param commandLine
	 * @return The next phase of the connection process.
	 */
	private int acceptRequest(String commandLine) {
		
		// Quit connection if commandLine is null (probably closed socket)
		if (commandLine == null) return ConnectionThread.quitConnection;
		
		String firstToken, secondToken;
		StringTokenizer tokenizedCommandLine = new StringTokenizer(commandLine);
		
		if (tokenizedCommandLine.hasMoreTokens()) {
			firstToken = tokenizedCommandLine.nextToken();
			
			// Check the quit command
			if (firstToken.equalsIgnoreCase("quit")) {
				return ConnectionThread.quitConnection;
			}
			
			// Check the info command
			if (firstToken.equalsIgnoreCase("info")) {
				if (tokenizedCommandLine.hasMoreTokens()) {
					secondToken = tokenizedCommandLine.nextToken();
					
					// Do some analysis
					int urlScoredPoints = this.urlWordSearch.doSearch(secondToken); 
	        		if (urlScoredPoints > ConfigurationParser.getIntValue("limit_url_points")) {
	        			this.toSocket.println("200 OBJ=1.0 GRP=1.0 MAX=1.0 AVG=1.0 (URL scored points exceeded the maximum value)");
	        		}
	        		else {
	        			
	        			UrlParser currentUrlParser = new UrlParser(secondToken);
	        			
	        			if (!this.dsManager.hasPornographyInfo(currentUrlParser) ||
	        				dsManager.isExpired(currentUrlParser)) {
	        				Logging.debug(this, "Performing object analysis: " + currentUrlParser.toString());
	        				float pornProbability = this.localAnalyzer.doAnalysis(currentUrlParser);
	        				if (pornProbability >= 0) {
	            				dsManager.addUrlInfo(currentUrlParser, pornProbability);
	        				}
	        				else
	        					Logging.debug(this, "Could not do analysis of: " + currentUrlParser.toString());
	        			}
	        			
	        			this.toSocket.println("200 OBJ=" + dsManager.getObjectPornProbability(currentUrlParser) + " (Missing more info. TODO.)");
	        			
	        		}
									
				}
				else
					this.toSocket.println("400 Missing URL in 'INFO' command.");
			}
			
			// Check the list command
			else if (firstToken.equalsIgnoreCase("list")) {
				if (tokenizedCommandLine.hasMoreTokens()) {
					secondToken = tokenizedCommandLine.nextToken();
					this.toSocket.println("450 LIST command unavailable. Work in progress.");
				}
				else
					this.toSocket.println("400 Missing path in 'LIST' command.");
			}
			
			// Check for other commands that are not allowed at the current phase.
			else if (firstToken.equalsIgnoreCase("user")) {
				this.toSocket.println("402 The current user is already set.");
			}
			else if (firstToken.equalsIgnoreCase("pass")) {
				this.toSocket.println("402 Already authenticated.");
			}
						
			// Invalid command.
			else
				this.toSocket.println("400 Invalid command.");
		}
		else
			this.toSocket.println("400 Invalid command.");
		
		// Keep the current phase
		return ConnectionThread.acceptRequests;
		
	}
	
}
