/*
 * UserAuthenticator.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			03 de janeiro de 2008
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.IPC;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.StringTokenizer;
import br.usp.ime.linux.lemes.ConfigurationParser;
import br.usp.ime.linux.lemes.Logging;

/**
 * Static class for user authentication purposes.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class UserAuthenticator {
	
	// Where we store users, passwords and permissions
	private static HashMap<String, String>	userPasswords;
	private static HashMap<String, String>	userCommands; 

	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("passwd_file");
	    
	}

	/**
	 * Load the passwd file to memory.
	 * 
	 * @return <b>true</b> if the passwd file was sucessfully loaded or <b>false</b> if not.
	 */
	public static boolean initialize() {
		
		// Init the user database
		userPasswords	= new HashMap<String, String>();
		userCommands	= new HashMap<String, String>();
		
		if (!ConfigurationParser.isDefined("passwd_file")) {
			Logging.warning("UserAuthenticator: Missing 'passwd_file' in configuration file.");
			return false;
		}
		
		int				lineNumber = 0;
		String			filename = ConfigurationParser.getStringValue("passwd_file"),
						fileLine, username, password;
		StringTokenizer	tokenizedFileLine;
		
		try {

			Logging.debug("UserAuthenticator: Opening passwd file '" + filename + "'.");

			FileReader localFileReader = new FileReader(filename);
			BufferedReader fileHandler = new BufferedReader(localFileReader);

			while ((fileLine = fileHandler.readLine()) != null) {
				lineNumber++;
				tokenizedFileLine = new StringTokenizer(fileLine, ":");
				if (tokenizedFileLine.hasMoreTokens()) {
					username = tokenizedFileLine.nextToken().toLowerCase().trim();
					if (tokenizedFileLine.hasMoreTokens()) {
						password = tokenizedFileLine.nextToken().trim();
						if (tokenizedFileLine.hasMoreTokens()) {
							userPasswords.put(username, password);
							userCommands.put(username, tokenizedFileLine.nextToken().toLowerCase().trim());
						}
						else {
							Logging.warning("UserAuthenticator: Missing allowed commands at line #" + lineNumber);
						}
					}
					else {
						Logging.warning("UserAuthenticator: Missing password at line #" + lineNumber);
					}
				}
				else {
					Logging.warning("UserAuthenticator: Wrong line format #" + lineNumber);
				}
			}
			
		}
		catch (FileNotFoundException e) {
			Logging.error("UserAuthenticator: File not found while creating WordSearch object: '" + filename + "'");
		}
		catch (IOException e) {
			Logging.error("UserAuthenticator: Error while reading from file: '" + filename + "' (" + e + ")");
		}
		catch (Exception e) {
			Logging.error("UserAuthenticator: Enexpected error: " + e);
		}
		
		return false;
		
	}
	
	/**
	 * A synonymous for initialize().
	 *  
	 * @return <b>true</b> if the passwd file was sucessfully loaded or <b>false</b> if not.
	 */
	public static boolean reload() {
		
		return initialize();
		
	}
	
	/**
	 * Check if the username is on database.
	 * 
	 * @param username The username.
	 * @return <b>true</b> if the user is in database or <b>false</b> if it isn't.
	 */
	public static boolean isValidUser(String username) {
		
		return userPasswords.containsKey(username);
		
	}
	
	/**
	 * Check if a pair "username" and "password" are valid.
	 * 
	 * @param username The username.
	 * @param password The password.
	 * @return <b>true</b> if the user is valid or <b>false</b> if it isn't.
	 */
	public static boolean checkPassword(String username, String password) {
		
		if (userPasswords.containsKey(username) &&
			userPasswords.get(username).equals(password))
				return true;
		
		return false;
		
	}
	
	/**
	 * Check if the user is authorized to run a command.
	 * 
	 * @param username The username
	 * @param command The command.
	 * @return <b>true</b> if the user is allowed or <b>false</b> if don't.
	 */
	public static boolean checkCommand(String username, String command) { 
		
		if (userCommands.containsKey(username)) {
			
			String commandsAllowed = userCommands.get(username); 
			
			// Asterisk is a wildcard for anything
			if (commandsAllowed.contains("*"))
				return true;
			
			// The word "all" is also a wildcard that matches anything
			if (commandsAllowed.equals("all"))
				return true;
			
			// If the command is in there we return true.
			if (commandsAllowed.equals(command)			||
				commandsAllowed.startsWith(command + ",")	||
				commandsAllowed.endsWith("," + command)	||
				commandsAllowed.contains("," + command + ","))
				return true;
		}
		
		return false;
	}
	
}
