/*
 * SkinDetector.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			19 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.ImageProcessing;

import java.io.*;
import java.awt.Color;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import br.usp.ime.linux.lemes.Logging;

/**
 * The SkinDetector object receives an image and do image analysis or
 * segmentation using it's predefined skin colors.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class SkinDetector {
	
	private float	skinPercent;
	private int		imageWidth,
					imageHeight;
	
	// TODO: This class should be modified to a static class.
	
	/**
	 * The class constructor using an byte[] object.
	 * 
	 * @param imageBytes The image data loaded into a byte[] object.
	 */
	public SkinDetector(byte[] imageBytes) {

		try {
			BufferedImage image = ImageIO.read(new ByteArrayInputStream(imageBytes));
			
			if (image == null) {
				Logging.failure(this, "BufferedImage is null.");
				this.skinPercent = -1;
			}
			else {
			
				this.imageWidth		= image.getWidth();
				this.imageHeight	= image.getHeight();
			
				int		x, y;
				float	skinPixels	= 0;
			
				for (y = 0; y < this.imageHeight; y++)
					for (x = 0; x < this.imageWidth; x++)
						if (this.isSkinColor(image.getRGB(x, y))) skinPixels++;
			
				this.skinPercent = skinPixels / (this.imageWidth * this.imageHeight);
				Logging.debug(this, "Found " + (int) skinPixels + " (" + this.skinPercent +
						"%) skin pixels in a image of " + this.imageWidth + "x" + this.imageHeight + ".");
				
			}
		}
		catch (IOException e) {
			Logging.failure(this, "Could not create the BufferedImage object.");
			this.skinPercent = -1;
		}
		
	}
	
	/**
	 * Check if the pixel is skin colored. This method converts the
	 * RGBA colors to the HSV color schema, so it's easier to select the
	 * skin using our empiric obtained values.
	 * 
	 * @param mixedRGB The pixel value (8-bit values for RGBA)
	 * @return <b>true</b> if it seems skin or <b>false</b> if don't. 
	 */
	private boolean isSkinColor(int mixedRGB) {
		
		Color	pixel = new Color(mixedRGB);
		
		float	red, green, blue,
				hue, saturation, value,
				min, max;
						
		// The RGB components in the interval [0, 1]
		red		= (float) pixel.getRed()	/ 255;
		green	= (float) pixel.getGreen() 	/ 255;
		blue	= (float) pixel.getBlue()	/ 255;
		
		min = red;
		if (green < min)	min = green;
		if (blue < min)		min = blue;
		
		max = red;
		if (green > max)	max = green;
		if (blue > max)		max = blue;
		
		// Hue
		if (max == min) hue = 0;
		else {
			if (max == red) {
				if (green >= blue) {
					hue = 60 * (green - blue) / (max - min);
				}
				else {
					hue = 60 * (green - blue) / (max - min) + 360;
				}
			}
			else {
				if (max == green) {
					hue = 60 * (blue - red) / (max - min) + 120;
				}
				else {
					hue = 60 * (red - green) / (max - min) + 240;
				}
			}
		}
		
		// Saturation
		if (max == 0) {
			saturation = 0;
		}
		else {
			saturation = (max - min) / max;
		}

		// Value (Brightness/Lightness)
		value = max;
		
		/*
		 * Here he define if it is skin or not.
		 * 
		 * The empiric values obtained by SquidPCB project.
		 * 
		 * Hue between 3 and 30 degrees
		 * Saturation between 5% and 90%
		 * Value between 10% and 98%
		 * 
		 */ 
		if (hue        >= 3    && hue        <= 30   && 
            saturation >= 0.05 && saturation <= 0.90 &&
            value      >= 0.10 && value      <= 0.98)
            	return true;
        else
        	return false;
		
	}
	
	/**
	 * Get the percentage of skin colored pixels in the image.
	 *  
	 * @return The percentage value between 0 and 1.
	 */
	public float getSkinPercent() {
		
		return this.skinPercent;
		
	}
	
	/**
	 * Get the width of the image used to create the object
	 *  
	 * @return The width of the image.
	 */
	public int getImageWidth() {
		
		return this.imageWidth;
		
	}
	
	/**
	 * Get the height of the image used to create the object
	 *  
	 * @return The height of the image.
	 */
	public int getImageHeight() {
		
		return this.imageHeight;
		
	}
	
}
