/*
 * Logging.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			28 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes;

import java.io.*;

/**
 * This class is used for error message logging and software debug.
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1.1
 */
public class Logging {

	// Class constants for external use. Small values are more
	// important than greater ones.
	public static final int Type_Panic      = 0;
	public static final int Type_Error      = 1;
	public static final int Type_Failure    = 2;
	public static final int Type_Warning    = 3;
	public static final int Type_Info       = 4;
	public static final int Type_Debug      = 5;
	public static final int Type_Trace		= 6;
	
	// Private constants for level description
	private static final String[] levelDescription =
		{"Panic", "Error", "Failure", "Warning", "Info", "Debug", "Trace"};
	
	// Default values for logging.
	private static int      screenLogLevel	=  1;
	private static int      fileLogLevel	= -1;

	// The file handler for messages logging.
	private static PrintWriter	logFileHandler;
	
	/**
	 * Set log level for screen and file logging.
	 * 
	 * @param level The log level.
	 */
	public static void setLogLevel(int level)
	{
		screenLogLevel    	= level;
		fileLogLevel      	= level;
	}

	/**
	 * Set log level for screen output.
	 * 
	 * @param level The log level.
	 */
	public static void setScreenLogLevel(int level)
 	{
	 	screenLogLevel    = level;
 	}
	
	/**
	 * Set log level for file logging.
	 * 
	 * @param level The log level.
	 */
 	public static void setFileLogLevel(int level)
 	{
 		fileLogLevel      = level;
 	}

 	/**
 	 * Define a file to receive the logged messages.
 	 * 
 	 * @param filename The filename with it's path.
 	 * @return <b>true</b> if the file could be opened or <b>false</b> if some error occurred.
 	 */
 	public static boolean setLogfile(String filename)
 	{
 		try {
         	logFileHandler      =
         		new PrintWriter(
         			new BufferedWriter(
         				new FileWriter(filename, true)));
         	return true;
     	}
     	catch (Exception e) {
         	logFileHandler      = null;
         	System.err.println("ERROR. Cannot open log file! (" + e + ")");
         	return false;
     	}
 	}

 	/**
 	 * Log a message with a determined log level.
 	 * 
 	 * @param messageLevel A constant value of the Logging class (Type_*).
 	 * @param messageText A text message to be logged.
 	 */
 	private static void log(int messageLevel, String messageText)
 	{
 		if (messageLevel <= screenLogLevel) {
         	System.err.println((System.currentTimeMillis() / 1000 ) + " " + levelDescription[messageLevel] + ": " + messageText);
     	}

     	if (logFileHandler != null && messageLevel <= fileLogLevel) {
         	logFileHandler.println((System.currentTimeMillis() / 1000 ) + " " + levelDescription[messageLevel] + ": " + messageText);
         	logFileHandler.flush();
     	}
     	
     	// If we have a panic. EXIT!
     	if (messageLevel == Type_Panic) System.exit(1);
 	}
 	
 	/**
 	 * Log a message from an object with a determined log level.
 	 * 
 	 * @param messageLevel A constant value of the Logging class (Type_*).
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	private static void log(int messageLevel, Object thisObject, String messageText) {
 		
 		String	className	= thisObject.getClass().getSimpleName();
 		int		objectId	= thisObject.hashCode();
 		
 		Logging.log(messageLevel, className + "[" + objectId + "]: " + messageText);
 		
 	}

 	/**
 	 * Log a panic message. (Warning! This kind of message invokes a System.exit(),
 	 * and should be used only when an unrecoverable error occurred.)
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void panic(String messageText) {
 		
 		Logging.log(Type_Panic, messageText);
 		
 	}
 	
 	/**
 	 * Log a panic message. (Warning! This kind of message invokes a System.exit(),
 	 * and should be used only when an unrecoverable error occurred.)
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void panic(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Panic, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log an error message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void error(String messageText) {
 		
 		Logging.log(Type_Error, messageText);
 		
 	}
 	
 	/**
 	 * Log an error message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void error(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Error, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log a failure message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void failure(String messageText) {
 		
 		Logging.log(Type_Failure, messageText);
 		
 	}
 	
 	/**
 	 * Log a failure message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void failure(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Failure, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log a warning message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void warning(String messageText) {
 		
 		Logging.log(Type_Warning, messageText);
 		
 	}
 	
 	/**
 	 * Log a warning message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void warning(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Warning, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log an info message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void info(String messageText) {
 		
 		Logging.log(Type_Info, messageText);
 		
 	}
 	
 	/**
 	 * Log an info message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void info(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Info, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log a debug message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void debug(String messageText) {
 		
 		Logging.log(Type_Debug, messageText);
 		
 	}
 	
 	/**
 	 * Log a debug message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void debug(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Debug, thisObject, messageText);
 		
 	}
 	
 	/**
 	 * Log a trace message.
 	 * 
 	 * @param messageText A text message to be logged.
 	 */
 	public static void trace(String messageText) {
 		
 		Logging.log(Type_Trace, messageText);
 		
 	}
 	
 	/**
 	 * Log a trace message.
 	 * 
 	 * @param thisObject The object that invoked the message.
 	 * @param messageText A text message to be logged.
 	 */
 	public static void trace(Object thisObject, String messageText) {
 		
 		Logging.log(Type_Trace, thisObject, messageText);
 		
 	}
 	
}