/*
 * SquidExternalACL.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			28 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.StringTokenizer;
import br.usp.ime.linux.lemes.IPC.ConnectionClient;

/**
 * This class is used to run the "Squid-cache external ACL helper" that will
 * connect to SquidPCB daemon using TCP sockets to perform queries about
 * URLs's pornography probability. 
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1a
 */
public class SquidExternalACL {
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("logfile",	"squidexternalacl.log");
		ConfigurationParser.setOptionName("config",		"squidexternalacl.cfg");
		ConfigurationParser.setOptionName("loglevel",	"3");
		ConfigurationParser.setOptionName("verbose",	"0");
		
		ConfigurationParser.setOptionNameCommandlineAlias("logfile",	"-L");
		ConfigurationParser.setOptionNameCommandlineAlias("config",		"-c");
		ConfigurationParser.setOptionNameCommandlineAlias("loglevel",	"-l");
		ConfigurationParser.setOptionNameCommandlineAlias("verbose",	"-v");
		
		ConfigurationParser.setOptionName("server_ip_address");
		ConfigurationParser.setOptionName("server_port_number", "7722");
		ConfigurationParser.setOptionName("username");
		ConfigurationParser.setOptionName("password");
		ConfigurationParser.setOptionName("max_object_pornography");
		ConfigurationParser.setOptionName("max_group_pornography");
		ConfigurationParser.setOptionName("max_path_pornography");
		ConfigurationParser.setOptionName("max_avg_pornography");
		ConfigurationParser.setOptionName("deny_message");
		
	}
	
	/**
	 * The main loop for this class. The Squid-cache external ACL helper is
	 * instantiated using this class that perform queries to the server and
	 * check for porn content.
	 * 
	 * @param args The command line arguments.
	 */
	public static void main(String commandLineOptions[]) {
		
		// Software announce and license info
		System.err.println("SquidPCB v0.1 - Copyright (C) 2007, 2008 Fernando Lemes da Silva (http://squidpcb.sf.net)\n" +
				"This program comes with ABSOLUTELY NO WARRANTY. This is free software,\n" +
				"and you are welcome to redistribute it under certain conditions.\n" +
				"Please read the LICENSE.TXT file for details or get the license at\n" +
				"http://www.gnu.org/licenses/gpl.txt\n");
		
		// Initialize ConfigurationParser and basic configs
		ConfigurationParser.initialize();
		ConfigurationParser.parseCommandLine(commandLineOptions);
		
		// Logging initialization
		Logging.setLogfile(ConfigurationParser.getStringValue("logfile"));
		Logging.setFileLogLevel(ConfigurationParser.getIntValue("loglevel"));
		Logging.setScreenLogLevel(ConfigurationParser.getIntValue("verbose"));

		// Read configs from file
		if (!ConfigurationParser.readFile(ConfigurationParser.getStringValue("config")))
			Logging.panic("SquidExternalACL: Config file not found!");

		// Prepare the connection to server
		ConnectionClient squidPCBServer = new ConnectionClient();
		InetAddress ip;
		int			port;
		String		username,
					password;
		
		try {
			
			if (!ConfigurationParser.isDefined("server_ip_address"))
				Logging.panic("SquidExternalACL: Undefined 'server_ip_address' in config file.");
			
			ip = InetAddress.getByName(ConfigurationParser.getStringValue("server_ip_address"));
			
			port = ConfigurationParser.getIntValue("server_port_number"); 
			if (port < 1 || port > 65535) {
				Logging.panic("SquidExternalACL: Invalid port number at config file: " + port);
			}
			
			username = ConfigurationParser.getStringValue("username");
			password = ConfigurationParser.getStringValue("password");	
			
			squidPCBServer = new ConnectionClient(ip, port, username, password);	
			
			// Check if the ConnectionClient is authenticated
			if (!squidPCBServer.isConnected()) {
				Logging.panic("SquidExternalACL: Could not authenticate with server.");
			}
			
			// Get filtering parameters from file
			float max_object_pornography	= ConfigurationParser.getFloatValue("max_object_pornography");
			if (max_object_pornography < 0 || max_object_pornography > 1) {
				Logging.warning("SquidExternalACL: max_object_pornography must be between 0 and 1.");
				max_object_pornography = 2; // This value will never be reached (ignored value)
			}
			float max_group_pornography	= ConfigurationParser.getFloatValue("max_group_pornography");
			if (max_group_pornography < 0 || max_group_pornography > 1) {
				Logging.warning("SquidExternalACL: max_group_pornography must be between 0 and 1.");
				max_group_pornography = 2; // This value will never be reached (ignored value)
			}
			float max_path_pornography	= ConfigurationParser.getFloatValue("max_path_pornography");
			if (max_path_pornography < 0 || max_path_pornography > 1) {
				Logging.warning("SquidExternalACL: max_path_pornography must be between 0 and 1.");
				max_path_pornography = 2; // This value will never be reached (ignored value)
			}
			float max_avg_pornography		= ConfigurationParser.getFloatValue("max_avg_pornography");
			if (max_avg_pornography < 0 || max_avg_pornography > 1) {
				Logging.warning("SquidExternalACL: max_avg_pornography must be between 0 and 1.");
				max_avg_pornography = 2; // This value will never be reached (ignored value)
			}
			
			// Get deny message to be returned
			String denyMessage;
			if (ConfigurationParser.isDefined("deny_message"))
				denyMessage = ConfigurationParser.getStringValue("deny_message");
			else
				denyMessage = new String("URL blocked by SquidPCB");
			
			// Start the main loop 
			BufferedReader 	keyboard = new BufferedReader(new InputStreamReader(System.in));
			String 			fromSquid, url;
			StringTokenizer	urlTokenized;
			boolean running = true;
			while (running) {
				
				if (!squidPCBServer.isConnected()) {
					squidPCBServer.close();
					squidPCBServer = new ConnectionClient(ip, port, username, password);
				}
				
				try {
					
					// Read line from standard input
					fromSquid = keyboard.readLine();
					if (fromSquid == null) {
						Logging.info("SquidExternalACL: EOF detected. Quitting.");
						running = false;
					}
					else {
						urlTokenized = new StringTokenizer(fromSquid);

						// Contatenate all tokens in one string
						if (urlTokenized.hasMoreTokens()) {

							// Add the "://" to the protocol name
							url = urlTokenized.nextToken().concat("://");

							// Add the subsequent tokens
							while (urlTokenized.hasMoreTokens())
								url = url.concat(urlTokenized.nextToken());
						}
						else
							url = new String("");


						squidPCBServer.getInfo(url);

						if (squidPCBServer.getPObj() >= max_object_pornography ||
								squidPCBServer.getPGrp() >= max_group_pornography  ||
								squidPCBServer.getPMax() >= max_path_pornography   ||
								squidPCBServer.getPAvg() >= max_avg_pornography) {

							// Deny the access
							System.out.println("ERR message=\"" + denyMessage + "\"");
						}
						else {
							// Authorize the access
							System.out.println("OK");
						}
					}
				}
				catch (IOException e) {
					Logging.panic("SquidExternalACL: IOError: " + e);
				}
			}
			
		}
		catch (UnknownHostException e) {
			Logging.panic("SquidExternalACL: Invalid IP address at config file: " +
					ConfigurationParser.getStringValue("server_ip_address"));
		}
		catch (SecurityException e) {
			Logging.panic("SquidExternalACL: Security exception using IP address: " +
					ConfigurationParser.getStringValue("server_ip_address"));
		}
		catch (Exception e) {
			Logging.panic("SquidExternalACL: Exception: " + e);
		}
		
	}
	
}
