/*
 * UrlObject.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			16 de novembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.UrlObjectAnalyzer;

import java.awt.image.BufferedImage;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.imageio.ImageIO;

import br.usp.ime.linux.lemes.Logging;
import br.usp.ime.linux.lemes.Datastructure.UrlParser;

/**
 * This class is responsible to retrieve the content of an UrlParser object
 * using a proxy server that needs to be set using the setProxy() method.
 * The data retrieved is stored in the current object in a byte[] vector that
 * may be accessed using getData().
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1
 */
public class UrlObject {

	// Class enum and constants
	private enum fileType {
		unknown, text, image
	}
	private static final int	bufferSize		= 8192;
	private static final int	maxObjectSize	= 1024*1024; // 1MB
	
	// Proxy settings
	private int				proxyServerPort		= 0;
	private InetAddress		proxyServerAddress	= null;
	
	// Object Data
	private UrlParser		objectUrl;
	private byte[] 			objectData;
	private int				objectLength;
	private fileType		currentFileType;
	
	
	/**
	 * Define the proxy options to retrieve the objects.
	 * 
	 * @param ipAddress The IP address of the proxy server.
	 * @param port The TCP port number that listen to the requests.
	 * @return <b>true</b> if the settings seems to be ok or <b>false<b> if any problem occured.
	 */
	public boolean setProxy(String ipAddress, int port) {
		
		try {
			
			this.proxyServerAddress 	= InetAddress.getByName(ipAddress);
			
			if (port < 0 || port > 65535) {
				Logging.error(this, "Invalid port number (out of range).");
				return false;
			}
			
			this.proxyServerPort = port;
			return true;
			
		}
		catch (UnknownHostException e) {
			Logging.error(this, "Cannot set proxy ip address. Reason: Unknown host.");
		}
		catch (SecurityException e) {
			Logging.error(this, "Cannot set proxy ip address. Reason: Security failure.");
		}
		return false;
		
	}

	/**
	 * Retrieve the requested URL. The data is retrieved from the proxy and is
	 * stored in this object. It can be accessed using the getData() method.
	 * 
	 * @param url The URL of the object.
	 * @return <b>true</b> if the request was sucessful or <b>false</b> if an error occured.
	 */
	public boolean retrieve(UrlParser url) {
		
		this.objectUrl	= url;
		this.currentFileType	= fileType.unknown;
		
		if (this.proxyServerPort == 0 || this.proxyServerAddress == null) {
			
			Logging.error(this, "Cannot retrieve url. Reason: Proxy server isn't set.");
			return false;
			
		}
		else {
			
			Logging.debug(this, "Trying to retrieve URL '" + url.getUrl() + "'.");
			try {
				
				// Open connection
				Socket connection 		= new Socket(this.proxyServerAddress, this.proxyServerPort);
				PrintStream toSocket	= new PrintStream(connection.getOutputStream());
				InputStream fromSocket	= connection.getInputStream();
				
				// Make the request
				toSocket.print("GET " + url.getUrl() + " HTTP/1.0\r\n");
				
				/* 
				 * TODO: Implement proxy authentication
				 * toSocket.print("Proxy-authorization: Basic <base64(user:pass)>\r\n");
				 */
				
				toSocket.print("\r\n");
				
				// Retrieve headers 
				String 			header = localReadLine(fromSocket),
								parameter, value;
				StringTokenizer tokenizedHeader;
				int				provisionedSize = 0;
				byte[]			provisionedBuffer;
				
				// Check if the request was sucessfull
				if (header.contains("HTTP/1.0") && header.contains("200")) {
					
					provisionedSize		= UrlObject.bufferSize;
					provisionedBuffer	= new byte[UrlObject.bufferSize];
					while (!header.equals("")) {
						header = localReadLine(fromSocket);
						tokenizedHeader = new StringTokenizer(header);
						if (tokenizedHeader.hasMoreTokens()) {
							parameter = tokenizedHeader.nextToken();
							
							// If the "Content-lenght:" header is set a provisioned buffer is created
							// with the perfect size.
							if (parameter.equalsIgnoreCase("content-length:") &&
								tokenizedHeader.hasMoreTokens()) {
								try {
									provisionedSize = Integer.parseInt(tokenizedHeader.nextToken());
									if (provisionedSize > UrlObject.maxObjectSize) {
										Logging.failure(this, "The object seems to have " + provisionedSize +
												"bytes, and is greater than the limit of " + UrlObject.maxObjectSize +
												" bytes. Skipping.");
										fromSocket.close();
										toSocket.close();
										connection.close();
										return false;
									}
									provisionedBuffer = new byte[provisionedSize];
									Logging.trace(this, "Creating buffer of " + provisionedSize + " bytes.");
								}
								catch (Exception e) {}
							}
							else if (parameter.equalsIgnoreCase("content-type:") &&
								tokenizedHeader.hasMoreTokens()) {
								value = tokenizedHeader.nextToken().toLowerCase();
								if (value.startsWith("text/")) {
									this.currentFileType = fileType.text;
								}
								else if (value.startsWith("image/")) {
									this.currentFileType = fileType.image;
								}
							}
							
						}
						
					}
					
					// Get the content. Retrieve data in local buffer.
					this.objectLength = 0;
					int		length, copyIndex;
					byte[]	buffer = new byte[bufferSize],
							newProvisionedBuffer;
					
					while ((length = fromSocket.read(buffer)) > 0) {
						
						// Duplicate the provisioned buffer.
						if (provisionedSize < (this.objectLength + length)) {
							if ((provisionedSize * 2) > UrlObject.maxObjectSize) {
								Logging.failure(this, "The object seems to be greater than the limit of " +
										UrlObject.maxObjectSize + " bytes. Skipping.");
								fromSocket.close();
								toSocket.close();
								connection.close();
								return false;
							}
							Logging.trace(this, "Duplicating buffer size (" + provisionedSize + "->" + provisionedSize*2 + ").");
							newProvisionedBuffer = new byte[provisionedSize * 2];
							for (copyIndex = 0; copyIndex < this.objectLength; copyIndex++)
								newProvisionedBuffer[copyIndex] = provisionedBuffer[copyIndex];
							provisionedBuffer = newProvisionedBuffer;
							provisionedSize *= 2;
						}
						
						// Add the received bytes to the provisioned buffer.
						for (copyIndex = 0; copyIndex < length; copyIndex++)
							provisionedBuffer[this.objectLength + copyIndex] = buffer[copyIndex];
						
						// Update object length
						this.objectLength += length;
					}
					
					if (this.objectLength < provisionedSize) {
						this.objectData = new byte[this.objectLength];
						for (copyIndex = 0; copyIndex < this.objectLength; copyIndex++)
							this.objectData[copyIndex] = provisionedBuffer[copyIndex];
					}
					else
						this.objectData = provisionedBuffer;
					
					Logging.debug(this, "   Retrieved object with " + this.objectLength + " bytes.");
					fromSocket.close();
					toSocket.close();
					connection.close();
					return true;
					
				}
				else {
					Logging.failure(this, "   HTTP error while retrieving object. Response: '" + header + "'");
				}
				
				fromSocket.close();
				toSocket.close();
				connection.close();
				
			}
			catch (IOException e) {
				Logging.failure(this, "IO error opening socket. (" + e + ")");
			}
			catch (SecurityException e) {
				Logging.error(this, "Security error opening socket. (" + e + ")");
			}
			
		}
		
		return false;
	}
	
	/**
	 * Return a line from an InputStream object.
	 * 
	 * @param inputStream The InputStream object.
	 * @return The line.
	 */
	private String localReadLine(InputStream inputStream) throws IOException {
		
		String	line			= new String("");
		byte	newLine			= '\n',
				carrierReturn	= '\r';
		byte[]	singleByte		= new byte[1];
		
		while (inputStream.read(singleByte) == 1 && singleByte[0] != newLine) {
			if (singleByte[0] != carrierReturn)
				line = line.concat(new String(singleByte));
		}
		
		return line;
		
	}
	
	/**
	 * Get the data from the retrieved object. No checks are done if an error occurred
	 * or there is no data.
	 * 
	 * @return The object data.
	 */
	public byte[] getData() {
		
		return this.objectData;
		
	}

	/**
	 * Retrieve the original URL of the object.
	 * 
	 * @return The URL.
	 */
	public String getUrl() {
		
		return this.objectUrl.getUrl();
		
	}
	
	/**
	 * Return the information if the object is a text.
	 * 
	 * @return <b>true</b> if is text or <b>false</b> if don't.
	 */
	public boolean isText() {
		
		// Check content-type header
		if (this.currentFileType == fileType.text)
			return true;
		
		if (this.currentFileType == fileType.unknown) {
			
			int index;
			// Search for some chars that cannot appears in text files
			for (index = 0; index < this.objectLength; index++) {
				if (this.objectData[index] <= 31)	return false;
				if (this.objectData[index] == 127)	return false;
			}
			this.currentFileType = fileType.text;
			return true;
			
		}
		
		return false;
		
	}

	/**
	 * Return the information if the object is an image.
	 * 
	 * @return <b>true</b> if is image or <b>false</b> if don't.
	 */
	public boolean isImage() {
		
		// Check content-type header
		if (this.currentFileType == fileType.image) {
			return true;
		}
		
		// Lets check for "magic numbers" in the file.
		String file = new String(this.objectData);
		
		// GIF, PNG and JPEG magic numbers
		if (file.startsWith("GIF87a") ||
			file.startsWith("GIF89a") ||
			file.startsWith("\211PNG\r\n\032\n") ||
			(file.startsWith("\377\330") && file.endsWith("\377\331"))) {
			
			this.currentFileType = fileType.image;
			return true;
		}
		
		try {
			
			// Check if Java can create a BufferedImage object. This is not the best way to
			// check if the data is an image anyway...
			// TODO: Make some improvements or maybe create the object and share it with other classes.
			BufferedImage image = ImageIO.read(new ByteArrayInputStream(this.objectData));
			if (image != null) return true;
			
		}
		catch (Exception e) {
			// Do nothing. This is not a problem that needs to be treated! :-D
		}
					
		return false;
		
	}
	
}
