/*
 * UrlObjectAnalyzer.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			01 de novembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.UrlObjectAnalyzer;

import br.usp.ime.linux.lemes.*;
import br.usp.ime.linux.lemes.Datastructure.*;
import br.usp.ime.linux.lemes.ImageProcessing.SkinDetector;
import br.usp.ime.linux.lemes.WordSearch.*;

/**
 * This class is used to perform any kind of url content analysis. It receives
 * an UrlParser object, retrieves the content using the UrlObject class and
 * do some analysis to check for porn content using a WordSearch object and
 * other methods from the package ImageProcessing. 
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 * @see br.usp.ime.linux.lemes.ImageProcessing
 * @see br.usp.ime.linux.lemes.WordSearch
 */
public class UrlObjectAnalyzer {

	private UrlObject 	localUrlObject;
	private WordSearch 	textBadwords;
	private int			pornLimiarText	= ConfigurationParser.getIntValue("limit_textfile_points");
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("text_badwords_file");
		ConfigurationParser.setOptionName("limit_textfile_points");
		ConfigurationParser.setOptionName("proxyserver_ipaddress");
	    ConfigurationParser.setOptionName("proxyserver_portnumber");
	    
	}
	
	/**
	 * Class constructor. When an object of this class is instantiated some
	 * parameters are read from the config file, so if you change the proxy
	 * parameters a new object must be instantiated.
	 * 
	 * @param textAnalyzer A WordSearch object for text analysis.
	 */
	public UrlObjectAnalyzer(WordSearch textAnalyzer) {
		
		this.localUrlObject = new UrlObject();
		this.localUrlObject.setProxy(ConfigurationParser.getStringValue("proxyserver_ipaddress"),
									 ConfigurationParser.getIntValue("proxyserver_portnumber"));
		this.textBadwords	= textAnalyzer;
		
	}

	/**
	 * Do the porn analysis of the UrlParser object.
	 * 
	 * @param url The URL of the object to be analysed.
	 * @return The porn probability of the object.
	 */
	public float doAnalysis(UrlParser url) {
		
		Logging.debug(this, "Retrieving url: " + url.getUrl());
		if (this.localUrlObject.retrieve(url)) {
			
			if (this.localUrlObject.isText())
				return doTextAnalysis();
			
			if (this.localUrlObject.isImage())
				return doImageAnalysis();
			
			Logging.debug(this, "No analysis could be done in url: " + url.getUrl());
			return -1;
			
		}
		else {
			Logging.failure(this, "Could not retrieve url: " + url.getUrl());
			return -1;
		}
		
	}
	
	/**
	 * Do the text analysis for the UrlObject data. The score points are
	 * normalized according to the length of the data to prevent very long
	 * texts with repeated low points words to reach the limiar value.
	 * 
	 * @return The probability of the text be a pornography content.
	 */
	private float doTextAnalysis() {
		
		Logging.debug(this, "Performing text analysis of url: " + this.localUrlObject.getUrl());
		String urlText = new String(this.localUrlObject.getData());
		
		int urlPornPoints = this.textBadwords.doSearch(urlText),
			normalize;
		
		// Here we define how the normalize factor is calculated.
		if (urlText.length() > 8192)
			normalize = (int) Math.floor(Math.log(urlText.length() / 4096) / Math.log(2)); 
		else
			normalize = 1;
		
		Logging.debug(this, "   Text analysis reported " + urlPornPoints +
				" points with normalize factor " + normalize +
				" (max normalized points is " + this.pornLimiarText + ").");
		
		if ((urlPornPoints / normalize) >= this.pornLimiarText)
			return 1;
		else
			return ((float) urlPornPoints / (normalize * this.pornLimiarText));
		
	}
	
	/**
	 * Do the image analysis for the UrlObject data. The value returned
	 * is a function applied to the skin colored pixels percent of the
	 * image.
	 * 
	 * @return The probability of the image be of a nude body.
	 */
	private float doImageAnalysis() {
		
		Logging.debug(this, "Performing image analysis of url: " + this.localUrlObject.getUrl());
		SkinDetector imageAnalysis = new SkinDetector(this.localUrlObject.getData());
		
		float	skinPercent = imageAnalysis.getSkinPercent(),
				pornProbability;
		
		if (skinPercent == -1)
			return -1;
		
		if (skinPercent < 0.5) {
			pornProbability = 4 * skinPercent * skinPercent * skinPercent;
		}
		else {
			pornProbability = 4 * (skinPercent - 1) * (skinPercent - 1) * (skinPercent - 1) + 1;
		}
		
		Logging.debug(this, "   Image analysis reported " +	skinPercent +
				"% of skin in the image. The associated probability is " +
				pornProbability + ".");
		
		return pornProbability;
		
	}
	
}
