/*
 * AutomataState.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			20 de setembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.WordSearch;

/**
 * This class is used to represent automata states that are used to perform
 * string matching with associated int values.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
class AutomataState {

	// Instance variables
	private int				stateValue;
	private AutomataState[]	lookupTable;
	private AutomataState 	defaultLetterState;
	private AutomataState 	defaultNonLetterState;

	/**
	 * Class constructor. Creates an AutomataState object only with it's
	 * associated value.
	 * 
	 * @param value An int value associated with the current state.
	 */
	public AutomataState (int value) {
		
		this(value, null, null);
		
	}
	
	/**
	 * Class constructor. Creates an AutomataState object with it's
	 * associated value, and two basic states for the "delta()" transiction
	 * function.
	 * 
	 * @param value An int value associated with the current state.
	 * @param defaultLetterState The default AutomataState to return when a
	 *        letter that is not in the table is submitted to delta()
	 *        function.
	 * @param defaultNonLetterState  The default AutomataState to return
	 *        when a NON-letter that is not in the table is submitted to
	 *        delta() function.
	 */
	public AutomataState (int value, AutomataState defaultLetterState, AutomataState defaultNonLetterState)	{
		
		this.stateValue 			= value;
		this.defaultLetterState 	= defaultLetterState;
		this.defaultNonLetterState	= defaultNonLetterState;
		
		// Initialize the AutomataState vector
		int index;
		this.lookupTable			= new AutomataState[256];
		for (index = 0; index < 256; index++)
			this.lookupTable[index] = null;		
		
	}

	/**
	 * Change the default AutomataState to return when a letter that is
	 * not in the table is submitted to delta() function.
	 * 
	 * @param state The AutomataState.
	 */
	public void setDefaultLetterState(AutomataState state) {
		
		this.defaultLetterState		= state;
		
	}
	
	/**
	 * Change the default AutomataState to return when a NON-letter that
	 * is not in the table is submitted to delta() function.
	 * 
	 * @param state The AutomataState.
	 */
	public void setDefaultNonLetterState(AutomataState state) {
		
		this.defaultNonLetterState 	= state;
		
	}
	
	/**
	 * Change the value associated to the current state.
	 * 
	 * @param value The int value associated with the current state.
	 */
	public void setStateValue(int value) {
		
		this.stateValue 			= value;
		
	}

	/**
	 * Retrieve the value associated to the current state.
	 * 
	 * @return The <b>int</b> value.
	 */
	public int getStateValue() {
		
		return this.stateValue;
		
	}
	
	/**
	 * Gets the AutomataState for a given char. If the AutomataState isn't
	 * defined a new object is created and assigned to the char value.
	 * 
	 * @param character The char used to match an entry in the current
	 *        AutomataState.
	 * @param defaultLetterState The default AutomataState to return when a
	 *        letter that is not in the table is submitted to delta()
	 *        function.
	 * @param defaultNonLetterState The default AutomataState to return
	 *        when a NON-letter that is not in the table is submitted to
	 *        delta() function.
	 * @return The requested AutomataState object.
	 * 
	 * @see java.lang.Character#isLetter(char)
	 */
	public AutomataState getStateForChar(char character, AutomataState defaultLetterState, AutomataState defaultNonLetterState) {
		
		// The cast of the char to an int value.
		int index = (int) character;
		
		// "Strange" characters are mapped to value 0. Maybe this should be modified.
		// TODO: Performance tests using another kind of object (like Vector or HashMap)
		if (index > 255) index = 0;
		
		// If the AutomataState doesn't exists we create one.
		if (this.lookupTable[index] == null) {
			this.lookupTable[index] = new AutomataState(0, defaultLetterState, defaultNonLetterState);
		}
		
		return this.lookupTable[index];
		
	}
	
	/**
	 * Gets the AutomataState for a given char. The difference between
	 * getStateForChar() and delta() is that the delta() method returns
	 * the defaultLetterState or defaultNonLetterState when the AutomataState
	 * for the current char isn't defined.
	 * 
	 * This class uses the Character.isLetter() method to decides if an
	 * character should be considered a letter or not.
	 * 
	 * @param character The char user to match an entry in the current
	 * 		  AutomataState.
	 * @param scoreCounter An object to add the current object state value if
	 * 		  the char isn't a letter and there is no AutomataState that
	 *        matches the char.
	 * @return The next AutomataState.
	 * 
	 * @see java.lang.Character#isLetter(char)
	 */
	public AutomataState delta(char character, ScoreBoard scoreCounter) {
		
		// The cast of the char to an int value.
		int index = (int) character;
		
		// "Strange" characters are mapped to value 0. Maybe this should be modified.
		// TODO: Performance tests using another kind of object (like Vector or HashMap)
		if (index > 255) index = 0;
		
		if (this.lookupTable[index] != null) {
			return this.lookupTable[index];
		}
		else {
			if (Character.isLetter(character)) {
				return this.defaultLetterState;
			}
			else {				
				/* If the supplied char is not a letter we add *
				 * some	score to the ScoreBoard object         */
				scoreCounter.add(this.stateValue);
				
				return this.defaultNonLetterState;				
			}
		}
		
	}
	
}
