/*
 * WordSearch.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			21 de setembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes.WordSearch;

import java.io.*;
import java.util.*;
import br.usp.ime.linux.lemes.Logging;

/**
 * The WordSearch class is used to perform linear text scanning and string
 * matching. With this class we can define some words to be scanned, each one
 * with an int value associated, so we can scan a very long text and get the
 * sum of the values for each word match.
 * 
 * @author Fernando Lemes da Silva
 * @version 0.1
 */
public class WordSearch {

	// Special AutomataStates created for each WordSearch object.
	private AutomataState	initialState;
	private AutomataState	invalidWord;
	
	/**
	 * Class constructor. Creates an empty WordSearch object.
	 */
	public WordSearch() {
		
		this.initialState 	= new AutomataState(0);
		this.invalidWord	= new AutomataState(0);
		
		this.initialState.setDefaultLetterState(this.initialState);
		this.initialState.setDefaultNonLetterState(this.initialState);
		
		this.invalidWord.setDefaultLetterState(this.invalidWord);
		this.invalidWord.setDefaultNonLetterState(this.initialState);
		
	}
	
	/**
	 * Class constructor. Creates an WordSearch object and open a file to
	 * feed the object. All the lines in the file should appear as
	 * [word][tab][points]
	 * 
	 * @param filename The filename to feed the WordSearch object.
	 */
	public WordSearch(String filename) {
		
		this();
		String fileLine, word, stringPoints;
		int lineNumber = 0, points;
		StringTokenizer lineParameters;
		
		try {
			if (filename == null) {
				Logging.error(this, "Missing filename for object feeding.");
			}
			else {
				Logging.debug(this, "Opening file '" + filename + "' for automata feeding.");
				
				FileReader localFileReader = new FileReader(filename);
				BufferedReader fileHandler = new BufferedReader(localFileReader);
				
				while ((fileLine = fileHandler.readLine()) != null) {
					lineNumber++;
					lineParameters = new StringTokenizer(fileLine, "\t");
					if (lineParameters.hasMoreTokens()) {
						word = lineParameters.nextToken().trim();
						if (lineParameters.hasMoreTokens()) {
							stringPoints = lineParameters.nextToken().trim();
							try {
								points = Integer.parseInt(stringPoints);
								this.addWord(word, points);
							}
							catch (NumberFormatException e) {
								Logging.warning(this, "Invalid value at line " + lineNumber + ": '" + stringPoints + "'");
							}
						}
						else {
							Logging.warning(this, "Wrong number of parameters at line #" + lineNumber);
						}
					}
					else {
						Logging.warning(this, "Wrong number of parameters at line #" + lineNumber);
					}
				}
				
			}
		}
		catch (FileNotFoundException e) {
			Logging.error(this, "File not found while creating WordSearch object: '" + filename + "'");
		}
		catch (IOException e) {
			Logging.error(this, "Error while reading from file: '" + filename + "' (" + e + ")");
		}
		catch (Exception e) {
			Logging.error(this, "Enexpected error: " + e);
		}
		
	}
	
	/**
	 * Add a word with it's value to the automata.
	 * 
	 * @param word The word.
	 * @param points The associated int value.
	 */
	public void addWord(String word, int points) {
		
		// Ignore string case.
		String localWord = word.toLowerCase();
		
		Logging.trace(this, "Adding word '" + localWord + "' to automata.");
		
		AutomataState currentState = this.initialState;
		int index, length = localWord.length();
		
		for (index = 0; index < length; index++) {
			currentState = currentState.getStateForChar(localWord.charAt(index), this.invalidWord, this.initialState);
		}
		
		// Set the points for this word
		currentState.setStateValue(points);
		
	}
	
	/**
	 * Scan the text and return the assigned points.
	 * 
	 * @param someText The text to be scanned.
	 * @return The scored points.
	 */
	public int doSearch(String someText) {
		
		// Ignore string case.
		String	localText = someText.toLowerCase();
		
		// Do logging for tracing.
		if (localText.length() <= 64) {
			Logging.trace(this, "Invoked method doAnalysis() to text '" + localText + "'.");
		}
		else {
			Logging.trace(this, "Invoked method doAnalysis() to text '" + localText.substring(0, 60) + "...'.");
		}
		
		
		/*
		 * Create a new counter to this method. Multiple threads can run the same
		 * mathod with no concurrency problems.
		 */
		
		ScoreBoard scoreCounter 	= new ScoreBoard();
		AutomataState currentState 	= this.initialState;
		
		int index, length = localText.length();
		
		// Do the scan
		for (index = 0; index < length; index++) {
			currentState = currentState.delta(localText.charAt(index), scoreCounter);
		}
		
		/*
		 * Add the value to the last string. Maybe it's a perfect match and
		 * there is no NON-letter after the current word matched.
		 */
		scoreCounter.add(currentState.getStateValue());
		
		Logging.trace(this, "   The scored value was " + scoreCounter.value() + " points.");
		
		return scoreCounter.value();
		
	}
	
}
